@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
 from collections import deque, defaultdict
 from heapq import heappop, heappush
@@ -11,7 +11,6 @@ from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 
 __all__ = ['Work', 'PredecessorSuccessor', 'Leveling']
-__metaclass__ = PoolMeta
 
 
 def intfloor(x):
@@ -19,6 +18,7 @@ def intfloor(x):
 
 
 class Work:
+    __metaclass__ = PoolMeta
     __name__ = 'project.work'
     predecessors = fields.Many2Many('project.predecessor_successor',
         'successor', 'predecessor', 'Predecessors',
@@ -40,7 +40,9 @@ class Work:
         states={
             'invisible': Eval('type') != 'task',
             }, depends=['type'])
-    duration = fields.Function(fields.Float('Duration'), 'get_function_fields')
+    duration = fields.Function(
+        fields.TimeDelta('Duration', 'company_work_time'),
+        'get_function_fields')
     early_start_time = fields.DateTime("Early Start Time", readonly=True)
     late_start_time = fields.DateTime("Late Start Time", readonly=True)
     early_finish_time = fields.DateTime("Early Finish Time", readonly=True)
@@ -48,7 +50,7 @@ class Work:
     actual_start_time = fields.DateTime("Actual Start Time")
     actual_finish_time = fields.DateTime("Actual Finish Time")
     constraint_start_time = fields.DateTime("Constraint Start Time")
-    constraint_finish_time = fields.DateTime("Constraint  Finish Time")
+    constraint_finish_time = fields.DateTime("Constraint Finish Time")
     early_start_date = fields.Function(fields.Date('Early Start'),
             'get_function_fields')
     late_start_date = fields.Function(fields.Date('Late Start'),
@@ -96,7 +98,7 @@ class Work:
         if 'duration' in names:
             all_works = cls.search([
                     ('parent', 'child_of', ids),
-                    ('active', '=', True)]) + works
+                    ])
             all_works = set(all_works)
 
             durations = {}
@@ -108,20 +110,20 @@ class Work:
                     leafs.add(work.id)
 
                 total_allocation = 0
+                effort = work.effort_duration or datetime.timedelta()
                 if not work.allocations:
-                    durations[work.id] = (work.effort or 0)
+                    durations[work.id] = effort
                     continue
                 for allocation in work.allocations:
                     total_allocation += allocation.percentage
-                durations[work.id] = ((work.effort or 0)
+                durations[work.id] = datetime.timedelta(
+                    seconds=effort.total_seconds()
                     / (total_allocation / 100.0))
 
             while leafs:
                 for work_id in leafs:
                     work = id2work[work_id]
                     all_works.remove(work)
-                    if not work.active:
-                        continue
                     if work.parent and work.parent.id in durations:
                         durations[work.parent.id] += durations[work_id]
                 next_leafs = set(w.id for w in all_works)
@@ -168,6 +170,12 @@ class Work:
                         })
                 break
 
+    @property
+    def hours(self):
+        if not self.duration:
+            return 0
+        return self.duration.total_seconds() / 60 / 60
+
     @classmethod
     def add_minutes(cls, company, date, minutes):
         minutes = int(round(minutes))
@@ -290,7 +298,7 @@ class Work:
                     if values[work]['early_start_time']:
                         early_finish_time = self.add_hours(work.company,
                                 values[work]['early_start_time'],
-                                work.duration)
+                                work.hours)
                 values[work]['early_finish_time'] = early_finish_time
 
                 # Propagate constraint_start on successors
@@ -310,7 +318,7 @@ class Work:
 
                 # if no sibling continue to walk up the tree
                 if not work2children.get(work.parent):
-                    if not work.parent in values:
+                    if work.parent not in values:
                         values[work.parent] = {}
                     parent = work.parent
 
@@ -328,7 +336,7 @@ class Work:
                 early_start = None
 
             # update values
-            if not work in values:
+            if work not in values:
                 values[work] = {}
             values[work]['early_start_time'] = early_start
 
@@ -370,7 +378,7 @@ class Work:
                     if values[work]['late_finish_time']:
                         late_start_time = self.add_hours(work.company,
                                 values[work]['late_finish_time'],
-                                -work.duration)
+                                -work.hours)
                 values[work]['late_start_time'] = late_start_time
 
                 # Propagate constraint_finish on predecessors
@@ -390,7 +398,7 @@ class Work:
 
                 # if no sibling continue to walk up the tree
                 if not work2children.get(work.parent):
-                    if not work.parent in values:
+                    if work.parent not in values:
                         values[work.parent] = {}
                     parent = work.parent
 
@@ -408,7 +416,7 @@ class Work:
                 late_finish = None
 
             # update values
-            if not work in values:
+            if work not in values:
                 values[work] = {}
             values[work]['late_finish_time'] = late_finish
 
@@ -499,7 +507,7 @@ class Work:
                     elif item[1] >= delay + sibling.duration:
                         overloaded.append(item)
                     else:
-                        #Succes!
+                        # Succes!
                         break
 
                 heappush(timeline,
@@ -537,17 +545,19 @@ class Work:
             self.parent.compute_dates()
 
     @classmethod
-    def write(cls, works, values):
-        super(Work, cls).write(works, values)
+    def write(cls, *args):
+        super(Work, cls).write(*args)
 
-        if 'effort' in values:
-            for work in works:
-                work.reset_leveling()
-        fields = ('constraint_start_time', 'constraint_finish_time',
-                  'effort')
-        if reduce(lambda x, y: x or y in values, fields, False):
-            for work in works:
-                work.compute_dates()
+        actions = iter(args)
+        for works, values in zip(actions, actions):
+            if 'effort' in values:
+                for work in works:
+                    work.reset_leveling()
+            fields = ('constraint_start_time', 'constraint_finish_time',
+                      'effort')
+            if reduce(lambda x, y: x or y in values, fields, False):
+                for work in works:
+                    work.compute_dates()
 
     @classmethod
     def create(cls, vlist):
@@ -581,11 +591,14 @@ class PredecessorSuccessor(ModelSQL):
             ondelete='CASCADE', required=True, select=True)
 
     @classmethod
-    def write(cls, pred_succs, values):
+    def write(cls, *args):
         Work = Pool().get('project.work')
-        super(PredecessorSuccessor, cls).write(pred_succs, values)
+        super(PredecessorSuccessor, cls).write(*args)
 
-        works = Work.browse(values.itervalues())
+        work_ids = [v for values in args[1::2]
+            for k, v in values.itervalues()
+            if k in ('predecessor', 'successor')]
+        works = Work.browse(work_ids)
         for work in works:
             work.reset_leveling()
         for work in works:
